import * as i0 from '@angular/core';
import { NgModule } from '@angular/core';
import { ArrayObjectHelper, ArrayUniqueProcessType } from '@devs-studio/array';
import * as moment from 'moment';
import { validateNumber } from '@devs-studio/sunat/dist/table/table-2';
import { validateSerie } from '@devs-studio/sunat/dist/annex/annex-5';
import parsePhoneNumber from 'libphonenumber-js';
import validator from 'validator';

class DevsValidatorsModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.0", ngImport: i0, type: DevsValidatorsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.1.0", ngImport: i0, type: DevsValidatorsModule }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.1.0", ngImport: i0, type: DevsValidatorsModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.0", ngImport: i0, type: DevsValidatorsModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [],
                    exports: []
                }]
        }] });

function checkArrayUnique(fields, processType = ArrayUniqueProcessType.CASEINSENSITIVE) {
    return (control) => {
        const items = control.value;
        if (!ArrayObjectHelper.checkDuplicates(items, fields, processType)) {
            return { unique: { value: control.value } };
        }
        return null;
    };
}

function _baseFunction(right, rule, callback) {
    return (control) => {
        const leftValue = Number(control.value);
        const formGroup = control.parent;
        if (formGroup) {
            const rightValue = Number(formGroup.get(right)?.value);
            if (!Number.isNaN(leftValue) && !Number.isNaN(rightValue) && !callback(leftValue, rightValue)) {
                // formGroup.get(right)?.updateValueAndValidity({ emitEvent: false });
                return { [rule]: true };
            }
        }
        return null;
    };
}
function isGreaterThanOrEqual(right) {
    return _baseFunction(right, 'greaterThanOrEqual', (left, right) => {
        return left >= right;
    });
}
function isLessThanOrEqual(right) {
    return _baseFunction(right, 'lessThanOrEqual', (left, right) => {
        return left <= right;
    });
}

function mustMatchValidator(controlName, matchingControlName) {
    return (formGroup) => {
        const control = formGroup.controls[controlName];
        const matchingControl = formGroup.controls[matchingControlName];
        if (matchingControl.errors && !matchingControl.errors['confirmedValidator']) {
            return;
        }
        if (control.value !== matchingControl.value) {
            matchingControl.setErrors({ mustMatch: true });
        }
        else {
            matchingControl.setErrors(null);
        }
    };
}

function isEnum(entity) {
    return (control) => {
        const value = control.value;
        if (value && !Object.values(entity).includes(value)) {
            return { isEnum: { value: control.value } };
        }
        return null;
    };
}

function _isValidDate(format) {
    return (control) => {
        const value = control.value;
        if (value && !moment(value, format, true).isValid()) {
            return { 'isValidDate': true };
        }
        return null;
    };
}
function isValidDate(format) {
    return _isValidDate(format);
}

function isValidIdentificationNumber(code_attribute) {
    return (control) => {
        const value = control.value;
        const formGroup = control.parent;
        if (formGroup) {
            const code = formGroup.get(code_attribute)?.value;
            if (!validateNumber(code, value)) {
                return { isValidIdentificationNumber: { value: control.value } };
            }
        }
        return null;
    };
}

function _checkAtLeast(inputs, field, value, quantity = 1) {
    var count = inputs.reduce((acc, cur) => {
        if (cur[field] === value) {
            return acc + 1;
        }
        else {
            return acc;
        }
    }, 0);
    return count === quantity;
}
function checkArrayAtLeast(field, value, quantity = 1) {
    return (control) => {
        const items = control.value;
        if (!_checkAtLeast(items, field, value, quantity)) {
            return { atLeast: { value: control.value } };
        }
        return null;
    };
}

function _checkAtLeast2(inputs, fn, quantity = 1) {
    var count = inputs.reduce((acc, cur) => {
        if (fn(cur)) {
            return acc + 1;
        }
        else {
            return acc;
        }
    }, 0);
    return count === quantity;
}
function checkArrayAtLeast2(fn, quantity = 1) {
    return (control) => {
        const items = control.value;
        if (!_checkAtLeast2(items, fn, quantity)) {
            return { atLeast: { value: control.value } };
        }
        return null;
    };
}

function isValidDocumentSerie(file_code_attribute, sunat_code_attribute, is_electronic_attribute) {
    return (control) => {
        const serie = control.value;
        const formGroup = control.parent;
        if (formGroup) {
            const file_code = formGroup.get(file_code_attribute)?.value;
            const sunat_code = formGroup.get(sunat_code_attribute)?.value;
            const is_electronic = formGroup.get(is_electronic_attribute)?.value;
            if (!validateSerie(file_code, sunat_code, serie, is_electronic)) {
                return { isValidDocumentSerie: { value: control.value } };
            }
        }
        return null;
    };
}

function isValidPhoneNumber(country) {
    return (control) => {
        const phoneNumber = control.value;
        if (!phoneNumber) {
            // Si el campo está vacío, consideramos que es válido
            return null;
        }
        try {
            // Intenta parsear el número de teléfono
            const phoneNumberInstance = parsePhoneNumber(phoneNumber, country);
            // Verifica si el número es válido
            if (!phoneNumberInstance)
                return { phoneNumber: true };
            const isValid = phoneNumberInstance.isValid();
            return isValid ? null : { phoneNumber: true };
        }
        catch (e) {
            // Si hay un error al parsear, consideramos que el número no es válido
            return { phoneNumber: true };
        }
    };
}

function isValidEmail(options) {
    return (control) => {
        const emailAddress = control.value;
        if (!emailAddress) {
            // Si el campo está vacío, consideramos que es válido
            return null;
        }
        try {
            // Intenta parsear el número de teléfono
            const isValid = validator.isEmail(emailAddress, options);
            return isValid ? null : { email: true };
        }
        catch (e) {
            // Si hay un error al parsear, consideramos que el número no es válido
            return { email: true };
        }
    };
}

class DevsValidators {
    static checkArrayUnique(fields, processType = ArrayUniqueProcessType.CASEINSENSITIVE) {
        return checkArrayUnique(fields, processType);
    }
    static checkArrayAtLeast(field, value, quantity = 1) {
        return checkArrayAtLeast(field, value, quantity);
    }
    static checkArrayAtLeast2(fn, quantity = 1) {
        return checkArrayAtLeast2(fn, quantity);
    }
    static isGreaterThanOrEqual(right) {
        return isGreaterThanOrEqual(right);
    }
    static isLessThanOrEqual(right) {
        return isLessThanOrEqual(right);
    }
    static mustMatchValidator(controlName, matchingControlName) {
        return mustMatchValidator(controlName, matchingControlName);
    }
    static isEnum(entity) {
        return isEnum(entity);
    }
    static isValidDate(format) {
        return isValidDate(format);
    }
    static isValidIdentificationNumber(controlName) {
        return isValidIdentificationNumber(controlName);
    }
    static isValidDocumentSerie(fileCodeControlname, sunatCodeControName, isElectronicControlName) {
        return isValidDocumentSerie(fileCodeControlname, sunatCodeControName, isElectronicControlName);
    }
    static isValidPhoneNumber(country) {
        return isValidPhoneNumber(country);
    }
    static isValidEmail(options = undefined) {
        return isValidEmail(options);
    }
}

/*
 * Public API Surface of validators
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DevsValidators, DevsValidatorsModule };
//# sourceMappingURL=devs-studio-angular-validators.mjs.map
